package io.zipcoder;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.google.common.base.CharMatcher;
import com.google.common.base.Splitter;

import static java.util.stream.Collectors.groupingBy;


public class ItemParser {

    int errorCounter = 0;

    public int getErrorCounter() {
        return errorCounter;
    }

    public Map<String, Map<Double, List<Item>>> getItemsMap(ArrayList<Item> items) {
        return items
                .stream()
                .collect(
                        groupingBy(
                                Item::getName,
                                groupingBy(
                                        Item::getPrice
                                )
                        )
                );
    }

    public ArrayList<String> parseRawDataIntoStringArray(String rawData){
        String fixedString = fixString(rawData);
        String stringPattern = "##";
        ArrayList<String> response = splitStringWithRegexPattern(stringPattern , fixedString);
        return response;
    }

    public Item parseStringIntoItem(String rawItem) {
//        try {
//            validateItem(rawItem);
//        } catch (ItemParseException e) {
//            //NO OP
//        }
        Map<String, String> itemMap = null;
        try {
            itemMap = itemStringToHashMap(rawItem);
        } catch (ItemParseException e) {
            errorCounter++;
            return null;
        }
        return createItem(itemMap);
    }

    private Item createItem(Map<String, String> itemMap) {
        return new Item(itemMap.get("name"), Double.parseDouble(itemMap.get("price")), itemMap.get("type"), itemMap.get("expiration"));
    }

//    public boolean validateItem(String rawItem) throws ItemParseException{
//        boolean isValid = false;
//        if (rawItem.contains("[:\\W]")) {
//            throw new ItemParseException();
//        } else isValid = true;
//        return isValid;
//    }

    public String toLowerCase(String rawString) {
        Matcher m = Pattern.compile("\\w").matcher(rawString);
        StringBuilder sb = new StringBuilder();

        int last = 0;
        while (m.find()) {
            sb.append(rawString, last, m.start());
            sb.append(m.group(0).toLowerCase());
            last = m.end();
        }
        sb.append(rawString.substring(last));

        return sb.toString();
    }

    public String fixCo0kies(String rawString) {
        Matcher m = Pattern.compile("co0kies").matcher(rawString);
        String fixedString = m.replaceAll("cookies");
        return fixedString;
    }

    public String fixString(String rawString) {
        String lowerCased = toLowerCase(rawString);
        String fixedString = fixCo0kies(lowerCased);
        return fixedString;
    }

    public ArrayList<String> findKeyValuePairsInRawItemData(String rawItem){
        String stringPattern = "[;|\\^]";
        ArrayList<String> response = splitStringWithRegexPattern(stringPattern , rawItem);
        //TODO
        return response;
    }

    private ArrayList<String> splitStringWithRegexPattern(String stringPattern, String inputString){
        return new ArrayList<>(Arrays.asList(inputString.split(stringPattern)));
    }

    private Map<String, String> itemStringToHashMap(String rawItem) throws ItemParseException {
        CharMatcher matcher = CharMatcher.anyOf(";^%*!@");
        Map<String,String> itemMap = Splitter.on(matcher).withKeyValueSeparator(':').split(rawItem);
        for (Map.Entry<String,String> entry : itemMap.entrySet()) {
            if ( entry.getValue().equals("")) {
                throw new ItemParseException();
            }
        }
        return itemMap;
    }

}
